import requests
from urllib.parse import urljoin
import os
from typing import Optional, Union
from agenticmem_commons.api_schema.retriever_schema import (
    SearchInteractionRequest,
    SearchInteractionResponse,
    SearchUserProfileRequest,
    SearchUserProfileResponse,
    GetInteractionsRequest,
    GetInteractionsResponse,
    GetUserProfilesRequest,
    GetUserProfilesResponse,
)

if os.environ.get("TEST_ENV", "False") == "True":
    BACKEND_URL = "http://127.0.0.1:8000"  # Local server for testing
else:
    BACKEND_URL = "http://agenticmem-test.us-west-2.elasticbeanstalk.com:8081"  # Elastic Beanstalk server url

from agenticmem_commons.api_schema.service_schemas import (
    InteractionRequest,
    ProfileChangeLogResponse,
    PublishUserInteractionRequest,
    PublishUserInteractionResponse,
    DeleteUserProfileRequest,
    DeleteUserProfileResponse,
    DeleteUserInteractionRequest,
    DeleteUserInteractionResponse,
)
from agenticmem_commons.api_schema.login_schema import Token


class AgenticMemClient:
    """Client for interacting with the AgenticMem API."""

    def __init__(self, api_key: str = ""):
        """Initialize the AgenticMem client.

        Args:
            api_key (str): Your API key for authentication
        """
        self.api_key = api_key
        self.base_url = BACKEND_URL
        self.session = requests.Session()

    def _make_request(
        self, method: str, endpoint: str, headers: Optional[dict] = None, **kwargs
    ):
        """Make an HTTP request to the API.

        Args:
            method (str): HTTP method (GET, POST, DELETE)
            endpoint (str): API endpoint
            headers (dict, optional): Additional headers to include in the request
            **kwargs: Additional arguments to pass to requests

        Returns:
            dict: API response
        """
        url = urljoin(self.base_url, endpoint)
        if self.api_key:
            self.session.headers.update(
                {
                    "Authorization": f"Bearer {self.api_key}",
                    "Content-Type": "application/json",
                }
            )
        if headers:
            self.session.headers.update(headers)
        response = self.session.request(method, url, **kwargs)
        response.raise_for_status()
        return response.json()

    def login(self, email: str, password: str) -> Token:
        """Login to the AgenticMem API.

        Args:
            email (str): The user's email
            password (str): The user's password

        Returns:
            Token: Response containing success status and message
        """
        response = self._make_request(
            "POST",
            "/token",
            data={"username": email, "password": password},
            headers={
                "Content-Type": "application/x-www-form-urlencoded",
                "accept": "application/json",
            },
        )
        return Token(**response)

    def publish_interaction(
        self,
        request_id: str,
        user_id: str,
        interaction_requests: list[Union[InteractionRequest, dict]],
    ) -> PublishUserInteractionResponse:
        """Publish user interactions.

        Args:
            request_id (str): The request ID
            user_id (str): The user ID
            interaction_requests (List[InteractionRequest]): List of interaction requests

        Returns:
            PublishUserInteractionResponse: Response containing success status and message
        """
        interaction_requests = [
            (
                InteractionRequest(**interaction_request)
                if isinstance(interaction_request, dict)
                else interaction_request
            )
            for interaction_request in interaction_requests
        ]
        request = PublishUserInteractionRequest(
            request_id=request_id,
            user_id=user_id,
            interaction_requests=interaction_requests,
        )
        response = self._make_request(
            "POST",
            "/api/publish_interaction",
            data=request.model_dump_json(),
        )
        return PublishUserInteractionResponse(**response)

    def search_interactions(
        self,
        request: Union[SearchInteractionRequest, dict],
    ) -> SearchInteractionResponse:
        """Search for user interactions.

        Args:
            request (SearchInteractionRequest): The search request

        Returns:
            SearchInteractionResponse: Response containing matching interactions
        """
        if isinstance(request, dict):
            request = SearchInteractionRequest(**request)
        response = self._make_request(
            "POST",
            "/api/search_interactions",
            data=request.model_dump_json(),
        )
        return SearchInteractionResponse(**response)

    def search_profiles(
        self,
        request: Union[SearchUserProfileRequest, dict],
    ) -> SearchUserProfileResponse:
        """Search for user profiles.

        Args:
            request (SearchUserProfileRequest): The search request

        Returns:
            SearchUserProfileResponse: Response containing matching profiles
        """
        if isinstance(request, dict):
            request = SearchUserProfileRequest(**request)
        response = self._make_request(
            "POST", "/api/search_profiles", data=request.model_dump_json()
        )
        return SearchUserProfileResponse(**response)

    def delete_profile(
        self, user_id: str, profile_id: str = "", search_query: str = ""
    ) -> DeleteUserProfileResponse:
        """Delete user profiles.

        Args:
            user_id (str): The user ID
            profile_id (str, optional): Specific profile ID to delete
            search_query (str, optional): Query to match profiles for deletion

        Returns:
            DeleteUserProfileResponse: Response containing success status and message
        """
        request = DeleteUserProfileRequest(
            user_id=user_id,
            profile_id=profile_id,
            search_query=search_query,
        )
        response = self._make_request(
            "DELETE", "/api/delete_profile", data=request.model_dump_json()
        )
        return DeleteUserProfileResponse(**response)

    def delete_interaction(
        self, user_id: str, interaction_id: str
    ) -> DeleteUserInteractionResponse:
        """Delete a user interaction.

        Args:
            user_id (str): The user ID
            interaction_id (str): The interaction ID to delete

        Returns:
            DeleteUserInteractionResponse: Response containing success status and message
        """
        request = DeleteUserInteractionRequest(
            user_id=user_id, interaction_id=interaction_id
        )
        response = self._make_request(
            "DELETE", "/api/delete_interaction", data=request.model_dump_json()
        )
        return DeleteUserInteractionResponse(**response)

    def get_profile_change_log(self) -> ProfileChangeLogResponse:
        response = self._make_request("GET", "/api/profile_change_log")
        return ProfileChangeLogResponse(**response)

    def get_interactions(
        self,
        request: Union[GetInteractionsRequest, dict],
    ) -> GetInteractionsResponse:
        """Get user interactions.

        Args:
            request (GetInteractionsRequest): The list request

        Returns:
            GetInteractionsResponse: Response containing list of interactions
        """
        if isinstance(request, dict):
            request = GetInteractionsRequest(**request)
        response = self._make_request(
            "POST",
            "/api/get_interactions",
            data=request.model_dump_json(),
        )
        return GetInteractionsResponse(**response)

    def get_profiles(
        self,
        request: Union[GetUserProfilesRequest, dict],
    ) -> GetUserProfilesResponse:
        """Get user profiles.

        Args:
            request (GetUserProfilesRequest): The list request

        Returns:
            GetUserProfilesResponse: Response containing list of profiles
        """
        if isinstance(request, dict):
            request = GetUserProfilesRequest(**request)
        response = self._make_request(
            "POST",
            "/api/get_profiles",
            data=request.model_dump_json(),
        )
        return GetUserProfilesResponse(**response)
